import java.io.*;
import java.util.*;

import org.apache.poi.xssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.*;
import org.apache.poi.openxml4j.opc.*;

public class TestCopyFileExcel
{
	public static void main(String[] args)
	{
		TestCopyFileExcel test = new TestCopyFileExcel();
		try{
			test.copy("TestTemplate.xlsx", "TestReport.xlsx");
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	public void copy(String templateName, String reportName) throws Exception
	{
		File templateFile = new File(templateName);
		if (!templateFile.exists()) {
		  throw new FileNotFoundException("The template '" + templateName + "' not found!");
		}

		XSSFWorkbook template = (XSSFWorkbook)WorkbookFactory.create(templateFile);

		File reportFile = new File(reportName);
		if(reportFile.exists() && !reportFile.delete()){
			throw new Exception("Failed to delete existing file " + reportName);
		}

		if(!reportFile.createNewFile()){
			throw new Exception("Failed to create new file " + reportName);
		}

		XSSFWorkbook report = new XSSFWorkbook();
		for(int i = 0; i < template.getNumberOfSheets(); i++)
		{
			XSSFSheet sourceSheet = template.getSheetAt(i);
			XSSFSheet destSheet = report.createSheet(sourceSheet.getSheetName());

			copySheet(sourceSheet, destSheet, report);

			int numMergedRegions = sourceSheet.getNumMergedRegions();
			for(int j = 0; j < numMergedRegions; j++)
			{
				CellRangeAddress range = sourceSheet.getMergedRegion(j);
				destSheet.addMergedRegion(new CellRangeAddress(range.getFirstRow(), range.getLastRow(),
										  					   range.getFirstColumn(), range.getLastColumn()));
			}
		}

		template.close();

		FileOutputStream fos = new FileOutputStream(reportFile);
		report.write(fos);
		fos.close();

		System.out.println("Success!");
	}

	public void copySheet(XSSFSheet sourceSheet, XSSFSheet destSheet, XSSFWorkbook destWb) throws Exception
	{
		for(Row sourceRow : sourceSheet)
		{
		    XSSFRow destRow = destSheet.createRow(sourceRow.getRowNum());
		    copyRow((XSSFRow)sourceRow, destRow, destWb);
		}
	}

	public void copyRow(XSSFRow sourceRow, XSSFRow destRow, XSSFWorkbook destWb) throws Exception
	{
		destRow.setHeight(sourceRow.getHeight());
		XSSFCellStyle sourceStyle = (XSSFCellStyle)sourceRow.getRowStyle();
		XSSFCellStyle destStyle = (XSSFCellStyle)destWb.createCellStyle();
		copyCellStyle(sourceStyle, destStyle, destWb);
		destRow.setRowStyle(destStyle);
		for(Cell sourceCell : sourceRow)
		{
		    XSSFCell destCell = destRow.createCell(sourceCell.getColumnIndex());
		    copyCell((XSSFCell)sourceCell, destCell, destWb);
		}
	}

	public void copyCell(XSSFCell sourceCell, XSSFCell destCell, XSSFWorkbook destWb) throws Exception
	{
	    int type = sourceCell.getCellType();
		destCell.setCellType(type);
		switch (type)
		{
		  case Cell.CELL_TYPE_FORMULA:
			destCell.setCellFormula(sourceCell.getCellFormula());
			break;
		  case Cell.CELL_TYPE_BOOLEAN:
			destCell.setCellValue(sourceCell.getBooleanCellValue());
			break;
		  case Cell.CELL_TYPE_NUMERIC:
			destCell.setCellValue(sourceCell.getNumericCellValue());
			break;
		  case Cell.CELL_TYPE_ERROR:
			destCell.setCellErrorValue(sourceCell.getErrorCellValue());
			break;
		  case Cell.CELL_TYPE_STRING:
			destCell.setCellValue(sourceCell.getRichStringCellValue());
			break;
		  case Cell.CELL_TYPE_BLANK:
		  default:
			break;
		}

		destCell.setCellComment(sourceCell.getCellComment());

		XSSFCellStyle sourceStyle = sourceCell.getCellStyle();
		XSSFCellStyle destStyle = destWb.createCellStyle();
		copyCellStyle(sourceStyle, destStyle, destWb);

		destCell.setCellStyle(destStyle);

		destCell.getSheet().setColumnWidth(sourceCell.getColumnIndex(), sourceCell.getSheet().getColumnWidth(sourceCell.getColumnIndex()));
	}

	public void copyCellStyle(XSSFCellStyle sourceStyle, XSSFCellStyle destStyle, XSSFWorkbook destWb) throws Exception
	{
		if(sourceStyle == null){
		    return;
		}

		destStyle.cloneStyleFrom(sourceStyle);
/*
		destStyle.setFillPattern(sourceStyle.getFillPatternEnum());

		// Copy background colors
		XSSFColor sf_clr = sourceStyle.getFillForegroundColorColor();
		if(sf_clr != null){
		    XSSFColor destColor = new XSSFColor(sf_clr.getCTColor());
		    destStyle.setFillForegroundColor(destColor);
		}

		XSSFColor sb_clr = sourceStyle.getFillBackgroundColorColor();
		if(sb_clr != null){
		    XSSFColor destColor = new XSSFColor(sb_clr.getCTColor());
		    destStyle.setFillBackgroundColor(destColor);
		}
*/
	}
}